#!/bin/bash
RED_COLOR='\e[1;31m'
GREEN_COLOR='\e[1;32m'
YELLOW_COLOR='\e[1;33m'
BLUE_COLOR='\e[1;34m'
PINK_COLOR='\e[1;35m'
SHAN='\e[1;33;5m'
RES='\e[0m'
clear

# Golang version
GOCLANG=1.19.2
# Node.js version
NODEJS=16.18.0

NOTE='\e[1;36m
==========================================================================
\r\n                        Alist 一键安装脚本\r\n
==========================================================================
\e[0m';
echo -e "$NOTE";

# Get platform
if [ -f /etc/os-release ] && [ $(grep -c openwrt /etc/os-release) -ne 0 ]; then
	# OpenWrt
    echo -e "\r\nOpenWrt 平台支持：${GREEN_COLOR}https://github.com/sbwml/luci-app-alist${RES}\r\n"
    sh -c "$(curl -sS https://raw.githubusercontent.com/sbwml/luci-app-alist/master/install.sh)"
    exit 0;
elif command -v arch >/dev/null 2>&1; then
	platform=`arch`
else
	platform=`uname -m`
fi

if [ "$platform" = "x86_64" ]; then
    ARCH=amd64
    [ -f "/etc/redhat-release" ] && centos_version=`cat /etc/redhat-release|sed -r 's/.* ([0-9]+)\..*/\1/'`
    [ $centos_version = 7 ] && ARCH="musl-amd64" && [ ! -f "/lib/ld-musl-x86_64.so.1" ] && INSTALL_LIBC=1
    NODEJSARCH=x64
elif [ "$platform" = "aarch64" ]; then
    ARCH=arm64
    NODEJSARCH=arm64
    #GLIBC_VERSION=`ldd --version | grep ldd | grep -oP '\) \K.*' | awk -F. '{print $2}'`
    GLIBC_VERSION=`ldd --version | awk '/ldd/ {print $NF}'`
    if [[ ! "$GLIBC_VERSION" -ge "28" ]]; then
        echo -e "\r\n${RED_COLOR}出错了${RES}，ARM64 架构要求 glibc 版本 ${GREEN_COLOR}≥ 2.28${RES}，当前 glibc 版本：${GREEN_COLOR}2.$GLIBC_VERSION${RES}"
        echo -e "ARM64 架构建议更换 Debian & Ubuntu 操作系统再试\r\n"
        exit 1;
    fi
fi

# Check root
if [ "$(id -u)" != "0" ]; then
    echo -e "\r\n${RED_COLOR}出错了，请使用 root 权限重试！${RES}\r\n" 1>&2
    exit 1;
elif [[ "$platform" != "x86_64" ]] && [[ "$platform" != "aarch64" ]]; then
    echo -e "\r\n${RED_COLOR}出错了${RES}，一键安装目前仅支持 x86_64 & aarch64 平台。\r\n其它平台请参考官方安装文档：${GREEN_COLOR}https://alist-doc.nn.ci${RES}\r\n"
    exit 1;
elif ! command -v systemctl >/dev/null 2>&1; then
    echo -e "\r\n${RED_COLOR}出错了${RES}，无法确定你当前的 Linux 发行版。\r\n建议参考官方安装文档：${GREEN_COLOR}https://alist-doc.nn.ci${RES}\r\n"
    exit 1;
else
    if command -v netstat >/dev/null 2>&1; then
        check_port=`netstat -lnp|grep 5244|awk '{print $7}'|awk -F/ '{print $1}'`
    else
        echo -e "${GREEN_COLOR}端口检查 ...${RES}"
        if command -v yum >/dev/null 2>&1; then
            yum install net-tools -y >/dev/null 2>&1
            check_port=`netstat -lnp|grep 5244|awk '{print $7}'|awk -F/ '{print $1}'`
        else
            apt-get update >/dev/null 2>&1
            apt-get install net-tools -y >/dev/null 2>&1
            check_port=`netstat -lnp|grep 5244|awk '{print $7}'|awk -F/ '{print $1}'`
        fi
    fi
fi

echo -e "\e[1;36m获取 Alist 版本信息 ...\e[0m"

# 获取公网IP
ip_info=`curl -s https://ip.cooluc.com`;
if [[ $disable_mirror = "yes" ]]; then
    isCN=NULL
else
    isCN=`echo $ip_info | grep -Po 'country_code\":"\K[^"]+'`;
fi
myip=`echo $ip_info | grep -Po 'ip\":"\K[^"]+'`;

# Github 镜像
if [ "$isCN" = "CN" ]; then
    ping -c 2 ghproxy.com > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        mirror="https://ghproxy.com/"
        git_mirror="https://hub.cooluc.com"
    else
        git_mirror="https://github.com"
    fi
fi

# 获取Alist版本
# EOL
latest_version="v1.0.6"
latest_version2="v2.6.4"

if [ "$isCN" = "CN" ]; then
    latest_version3=`curl -s "https://api.cooluc.com/alist/version3"`;
else
    latest_version3=`curl -s "https://api.github.com/repos/alist-org/alist/releases" | grep "tag_name" | grep v3 | head -n 1|awk -F ":" '{print $2}' | sed 's/\"//g;s/,//g;s/ //g'`;
fi

# Get latest commits of main
REMOTE_ALIST_COMMIT=`curl -s "https://api.cooluc.com/alist/commit"`;

# 
if [ -z "$latest_version3" ]; then
    echo "获取 alist 版本失败"
    exit 1
fi

CHECK() (
if [ $check_port ]; then
    kill -9 $check_port
fi
if [ ! -d "/opt/alist/" ]; then
    mkdir -p /opt/alist
fi
)

INSTALL() (
    if [ -f "/opt/alist/alist" ]; then
        INSTALLED
        exit 3
    fi

    # 下载 Alist 后端程序
    echo -e "\r\n${GREEN_COLOR}下载 Alist $latest_version ...${RES}"
    curl -Lk "$mirror"https://github.com/alist-org/alist/releases/download/$latest_version/alist_"$latest_version"_linux_$ARCH.tar.gz -o /tmp/alist.tar.gz $CURL_BAR
    tar zxf /tmp/alist.tar.gz -C /opt/alist/
    if [ -d /opt/alist/linux_$ARCH ]; then
        mv /opt/alist/linux_$ARCH/alist /opt/alist/
        rm -rf /opt/alist/linux_$ARCH
    else
        echo -e "${RED_COLOR}下载 alist_"$latest_version"_linux_"$ARCH".tar.gz 失败！${RES}"
        exit 1;
    fi

    # 下载 Alist 前端web
    echo -e "${GREEN_COLOR}下载 Alist-web $latest_version ...${RES}"
    curl -Lk "$mirror"https://github.com/alist-org/web-v2/releases/download/"$latest_version"/refs.tags."$latest_version".tar.gz -o /tmp/alist-web.tar.gz $CURL_BAR
    tar zxf /tmp/alist-web.tar.gz -C /opt/alist/
    if [ ! -d /opt/alist/dist ]; then
        echo -e "\r\n${RED_COLOR}下载 refs.tags."$latest_version".tar.gz 失败！${RES}"
        exit 1;
    fi

    # 删除下载缓存
    rm -rf /tmp/alist.tar.gz /tmp/alist-web.tar.gz
)

##################  V2  ##################
INSTALL_V2() (
    if [ -f "/opt/alist/alist" ]; then
        INSTALLED
        exit 3
    fi

    # 下载 Alist 程序
    echo -e "\r\n${GREEN_COLOR}下载 Alist $latest_version2 ...${RES}"
    curl -Lk "$mirror"https://github.com/alist-org/alist/releases/download/"$latest_version2"/alist-linux-amd64.tar.gz -o /tmp/alist.tar.gz $CURL_BAR
    tar zxf /tmp/alist.tar.gz -C /opt/alist/

    if [ -f /opt/alist/alist-linux-$ARCH ]; then
        mv /opt/alist/alist-linux-$ARCH /opt/alist/alist
    else
        echo -e "${RED_COLOR}下载 alist-linux-$ARCH.tar.gz 失败！${RES}"
        exit 1;
    fi

    # 删除下载缓存
    rm -rf /tmp/alist*
)
##################  V2 end  ##################

##################  V3  ##################
INSTALL_V3() (
    if [ -f "/opt/alist/alist" ]; then
        INSTALLED
        exit 3
    fi

    # 下载 Alist 程序
    echo -e "\r\n${GREEN_COLOR}下载 Alist $latest_version3 ...${RES}"
    # 安装 musl 运行库，兼容基于更高版本的 musl 编译器
	[ "$INSTALL_LIBC" == "1" ] && curl -s https://repo.cooluc.com/mailbox.repo > /etc/yum.repos.d/mailbox.repo && yum install -y musl-libc
    curl -Lk "$mirror"https://github.com/alist-org/alist/releases/download/"$latest_version3"/alist-linux-$ARCH.tar.gz -o /tmp/alist.tar.gz $CURL_BAR
    tar zxf /tmp/alist.tar.gz -C /opt/alist/

    if [ ! -f /opt/alist/alist ]; then
        echo -e "${RED_COLOR}下载 alist-linux-$ARCH.tar.gz 失败！${RES}"
        exit 1;
    fi

    # 删除下载缓存
    rm -rf /tmp/alist*
)
##################  V3 end  ##################


INSTALL_V3_BUILD() (
    echo -e "\r\n ${YELLOW_COLOR}注意：\r\n    1、安装开发版要求系统可用内存大于 2GB 或更多\r\n    2、部分国内服务器因网络缘故可能无法正确安装（遇此情况建议安装选项 1 or 2）${RES}\r\n"
    echo -e " ${YELLOW_COLOR}键入${RES} ${GREEN_COLOR}Ctrl C${RES} ${YELLOW_COLOR}可终止操作${RES}"
    echo -ne " ${SHAN}按回车继续 ↵ ${RES}"
    read
    echo -e "\r\n${GREEN_COLOR}正在安装 Alist 开发版 ...${RES}"

    # git 检查
    if ! command -v git >/dev/null 2>&1; then
        if command -v yum >/dev/null 2>&1; then
            yum install git -y
        else
            apt-get update
            apt-get install git -y
        fi
    fi

    # GCC 检查
    if ! command -v gcc >/dev/null 2>&1; then
        if command -v yum >/dev/null 2>&1; then
            yum install gcc gcc-c++ -y
        else
            apt-get update
            apt-get install gcc g++ -y
        fi
    fi

    # 下载 Golang
    echo -e "\r\n${GREEN_COLOR}下载 Go v$GOCLANG ...${RES}"
    curl -L https://dl.google.com/go/go$GOCLANG.linux-$ARCH.tar.gz -o /tmp/go$GOCLANG.linux-$ARCH.tar.gz $CURL_BAR
    tar zxf /tmp/go$GOCLANG.linux-$ARCH.tar.gz -C /tmp/
    mkdir /tmp/go/tmp
    export PATH="/tmp/go/bin:$PATH"
    export GOPATH="/tmp/go/tmp"

    # 根据地域设置 GOPROXY 镜像源
    if [ $isCN = "CN" ]; then
        export GO111MODULE=on
        export GOPROXY="https://goproxy.cn"
    fi

    # 下载 Nodejs
    echo -e "\r\n${GREEN_COLOR}下载 Nodejs v$NODEJS ...${RES}"
    if [ $isCN = "CN" ]; then
        #curl -L https://npm.taobao.org/mirrors/node/v$NODEJS/node-v$NODEJS-linux-$NODEJSARCH.tar.xz -o /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz $CURL_BAR
        curl -Lk https://registry.npmmirror.com/-/binary/node/v$NODEJS/node-v$NODEJS-linux-$NODEJSARCH.tar.xz -o /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz $CURL_BAR
    else
        curl -Lk https://nodejs.org/dist/v$NODEJS/node-v$NODEJS-linux-$NODEJSARCH.tar.xz -o /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz $CURL_BAR
    fi
    tar xf /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz -C /tmp/
    export PATH="/tmp/node-v$NODEJS-linux-$NODEJSARCH/bin:$PATH"

    # 根据地域设置 npm 镜像源
    if [ $isCN = "CN" ]; then
        npm config set registry https://registry.npm.taobao.org
    fi

    # clone alist-web source
    echo -e "\r\n${GREEN_COLOR}下载 Alist-web 源码 ...${RES}\r\n"
    rm -rf /tmp/alist-web
    git clone $git_mirror/alist-org/alist-web -b main --recurse-submodules --depth=1 /tmp/alist-web
    if [ $? = 0 ]; then
        cd /tmp/alist-web
    else
        echo -e "\r\n${RED_COLOR}出错了${RES}，Alist-web 编译源码下载失败"
        echo -e "\r\n${GREEN_COLOR}你可以使用脚本 1 或 2 选项进行安装${RES}"
        rm -rf /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz /tmp/go$GOCLANG.linux-$ARCH.tar.gz /tmp/alist* /tmp/go /tmp/node-* /tmp/yarn--*
        exit 1
    fi

    echo -e "\r\n${GREEN_COLOR}正在编译 Alist-web ...${RES}"
    npm install -g pnpm
    pnpm install
    #pnpm i18n:release
    pnpm build
    if [ ! -d "dist" ]; then
        echo -e "\r\n${RED_COLOR}出错了${RES}，Alist-web 编译失败，请确保系统内存大于 2GB 或更多"
        echo -e "\r\n${GREEN_COLOR}你可以使用脚本 1 或 2 选项进行安装${RES}"
        rm -rf /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz /tmp/go$GOCLANG.linux-$ARCH.tar.gz /tmp/alist* /tmp/go /tmp/node-* /tmp/yarn--*
        exit 1
    fi

    # clone alist source
    rm -rf /tmp/alist
    echo -e "\r\n${GREEN_COLOR}下载 Alist 源码 ...${RES}\r\n"
    git clone $git_mirror/alist-org/alist -b main --depth=1 /tmp/alist
    if [ $? = 0 ]; then
        cd /tmp/alist
    else
        echo -e "\r\n${RED_COLOR}出错了${RES}，Alist 编译源码下载失败"
        echo -e "\r\n${GREEN_COLOR}你可以使用脚本 1 或 2 选项进行安装${RES}"
        rm -rf /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz /tmp/go$GOCLANG.linux-$ARCH.tar.gz /tmp/alist* /tmp/go /tmp/node-* /tmp/yarn--*
        exit 1
    fi

    echo -e "\r\n${GREEN_COLOR}正在编译 Alist ...${RES}\r\n"
    appName="alist"
    builtAt="$(date +'%F %T %z')"
    goVersion=$(go version | sed 's/go version //')
    gitAuthor=$(git show -s --format='format:%aN <%ae>' HEAD)
    gitCommit=$(git log --pretty=format:"%h" -1)
    ldflags="\
-w -s \
-X 'github.com/alist-org/alist/v3/internal/conf.BuiltAt=$builtAt' \
-X 'github.com/alist-org/alist/v3/internal/conf.GoVersion=$goVersion' \
-X 'github.com/alist-org/alist/v3/internal/conf.GitAuthor=$gitAuthor' \
-X 'github.com/alist-org/alist/v3/internal/conf.GitCommit=$gitCommit' \
-X 'github.com/alist-org/alist/v3/internal/conf.Version=v3-$gitCommit' \
-X 'github.com/alist-org/alist/v3/internal/conf.WebVersion=v3-$gitCommit' \
"

    # 导入前端资源
    cp -r ../alist-web/dist/* public/dist/

    # 后端编译
    go build -ldflags="$ldflags" .
    if [ -f "alist" ]; then
        strip alist
        \cp -f alist /opt/alist/
    fi

    # 删除下载缓存
    rm -rf /tmp/node-v$NODEJS-linux-$NODEJSARCH.tar.xz /tmp/go$GOCLANG.linux-$ARCH.tar.gz /tmp/alist* /tmp/go /tmp/node-* /tmp/yarn--*
)

INIT() (
    if [ ! -f "/opt/alist/alist" ]; then
        echo -e "\r\n${RED_COLOR}出错了${RES}，当前系统未安装 Alist\r\n"
        exit 1;
    else
        rm -f /opt/alist/alist.db
        systemctl stop alist >/dev/null 2>&1
    fi
    if [[ $VERSION = "v1" ]]; then
        echo -ne "\r\n${BLUE_COLOR}请输入阿里云盘 refresh_token：${RES}"
        read REFRESH_TOKEN
        cat > /opt/alist/conf.yml <<EOF
info:
  title: AList
  logo: ""
  footer_text: Xhofe's Blog
  footer_url: https://www.nn.ci
  music_img: https://img.oez.cc/2020/12/19/0f8b57866bdb5.gif
  check_update: true
  script:
  autoplay: true
  sort: name-descend
  preview:
    text: [txt,htm,html,xml,java,properties,sql,js,md,json,conf,ini,vue,php,py,bat,gitignore,yml,go,sh,c,cpp,h,hpp]
server:
  address: "0.0.0.0"
  port: "5244"
  search: true
  download: true
  static: dist
  site_url: '*'
  password: password
ali_drive:
  api_url: https://api.aliyundrive.com/v2
  max_files_count: 50
  drives:
  - refresh_token: $REFRESH_TOKEN
    root_folder: root
    name: home
    password: 
    hide: false
database:
  type: sqlite3
  dBFile: alist.db

EOF
    fi

    # 创建 systemd
    if [[ $VERSION = "v3" ]]; then
        cat >/lib/systemd/system/alist.service <<EOF
[Unit]
Description=Alist service
Wants=network.target
After=network.target network.service

[Service]
Type=simple
WorkingDirectory=/opt/alist
ExecStart=/opt/alist/alist server
Restart=on-failure
RestartPreventExitStatus=23

[Install]
WantedBy=multi-user.target
EOF
    else
        cat >/lib/systemd/system/alist.service <<EOF
[Unit]
Description=Alist service
Wants=network.target
After=network.target network.service

[Service]
Type=simple
WorkingDirectory=/opt/alist
ExecStart=/opt/alist/alist
Restart=on-failure
RestartPreventExitStatus=23

[Install]
WantedBy=multi-user.target
EOF
    fi

    # 添加开机启动
    systemctl daemon-reload
    systemctl enable alist >/dev/null 2>&1
    systemctl start alist

    # 防火墙放行 Alist 服务端口
    echo
    echo -e "${GREEN_COLOR}本地防火墙放行 5244 端口${RES}"
    if [ -f /usr/lib/systemd/system/iptables.service ]; then  # 如果服务不存在，跳过状态判断，避免下文抛出多余日志
        IPTABLES_STATUS=`systemctl status iptables`
    fi
    FIREWALLD_STATUS=`systemctl status firewalld`
    if [[ "$IPTABLES_STATUS" =~ "active" ]]; then
        WORK_PORT=`iptables-save | grep ACCEPT | grep 5244`
        if [[ -z $WORK_PORT ]]; then
            iptables -A INPUT -p tcp -m tcp --dport 5244 -j ACCEPT
            service iptables save
        fi
    elif [[ "$FIREWALLD_STATUS" =~ "active" ]]; then
        WORK_PORT=`firewall-cmd --list-all --zone public | grep 5244/tcp`
        if [[ -z $WORK_PORT ]]; then
            firewall-cmd --zone=public --add-port=5244/tcp --permanent
            firewall-cmd --reload
        fi
    fi

    if [[ $VERSION = "v1" ]]; then
    echo -e "${GREEN_COLOR}创建根目录缓存...${RES}"
    sleep 5 # 睡眠5秒，防止 Alist 启动后初始化未完成导致重建列表失败
    curl -d '{"path":"home","password":"password","depth":1}' -H "Content-Type: application/json" -X POST http://127.0.0.1:5244/api/rebuild >/dev/null 2>&1
    fi
)

SUCCESS() (
    clear
    echo -e "\r\n$NOTE\r\n";
    echo "Alist 安装成功！"
    echo -e "\r\n访问地址：${GREEN_COLOR}http://$myip:5244/${RES}\r\n"

    if [[ $VERSION = "v2" ]]; then
        echo -e "配置文件：${GREEN_COLOR}/opt/alist/data/config.json${RES}"
        pushd /opt/alist/ >/dev/null 2>&1
            get_password=`./alist -password 2>&1`
            local_password=`echo $get_password | awk -F'password: ' '{print $2}'`
        popd >/dev/null 2>&1
        echo -e "默认管理密码：${GREEN_COLOR}$local_password${RES}"
    elif [[ $VERSION = "v3" ]]; then
        echo -e "配置文件：${GREEN_COLOR}/opt/alist/data/config.json${RES}"
        pushd /opt/alist/ >/dev/null 2>&1
            get_password=`./alist password 2>&1 | tail -2`
            local_username=`echo "$get_password" | awk '{print $2}' | head -1`
            local_password=`echo "$get_password" | awk '{print $2}' | tail -1`
        popd >/dev/null 2>&1
        echo -e "用户名：\e[1;36m$local_username\e[0m"
        echo -e "密码：\e[1;36m$local_password\e[0m"
    else
        echo -e "配置文件：${GREEN_COLOR}/opt/alist/conf.yml${RES}"
        echo -e "重构目录密码：${GREEN_COLOR}password${RES}"
    fi

    echo
    echo -e "查看状态：${GREEN_COLOR}systemctl status alist${RES}"
    echo -e "启动服务：${GREEN_COLOR}systemctl start alist${RES}"
    echo -e "重启服务：${GREEN_COLOR}systemctl restart alist${RES}"
    echo -e "停止服务：${GREEN_COLOR}systemctl stop alist${RES}"
    echo -e "\r\n温馨提示：如果端口无法正常访问，请检查 \e[1;36m服务器安全组、本机防火墙、Alist状态\e[0m"
    echo
)

UNINSTALL() (
    echo -e "\r\n${GREEN_COLOR}卸载 Alist ...${RES}\r\n"
    echo -e "${GREEN_COLOR}停止进程${RES}"
    systemctl disable alist >/dev/null 2>&1
    systemctl stop alist >/dev/null 2>&1
    echo -e "${GREEN_COLOR}清除残留文件${RES}"
    rm -rf /opt/alist /lib/systemd/system/alist.service
    systemctl daemon-reload
    echo -e "\r\n${GREEN_COLOR}Alist 已在系统中移除！${RES}\r\n"
)

INSTALLED() (
    echo -e "\r\n 当前已安装 Alist 版本：${GREEN_COLOR}$local_alist_version${RES}"
    echo -e " 你可以进行${GREEN_COLOR} 更新 ${RES}或${RED_COLOR} 卸载 ${RES}操作\r\n"
)

UPDATE() (
    if [ ! -f "/opt/alist/alist" ]; then
        echo -e "\r\n${RED_COLOR}出错了${RES}，当前系统未安装 Alist\r\n"
        exit 1;
    else
        echo
        if [ -f "/opt/alist/conf.yml" ]; then
            echo -e " ${YELLOW_COLOR}注意：当前已安装 Alist v1 版本，更新 v3 版本将会覆盖原有配置参数${RES}"
            echo -e " ${YELLOW_COLOR}你可以键入${RES} ${BLUE_COLOR}Ctrl C${RES} ${YELLOW_COLOR}组合按钮终止操作${RES}"
            echo -ne "\r\n ${SHAN}按回车继续更新${RES}"
            read
            echo -e "\r\n${GREEN_COLOR}停止 Alist 进程${RES}"
            systemctl stop alist
            rm -rf /opt/alist/*
            VERSION=v3
            INSTALL_V3
            INIT >/dev/null 2>&1
            echo -e "\r\n${GREEN_COLOR}启动 Alist 进程${RES}"
            systemctl start alist
            echo -e "\r\n${GREEN_COLOR}Alist 已更新到最新稳定版！${RES}\r\n"
            exit 0;
        elif [ -f "/opt/alist/data/config.json" ] && [[ $(grep -c assets "/opt/alist/data/config.json") -ne 0 ]]; then
            echo -e " ${YELLOW_COLOR}注意：当前已安装 Alist v2 版本，更新 v3 版本将会覆盖原有配置参数${RES}"
            echo -e " ${YELLOW_COLOR}你可以键入${RES} ${BLUE_COLOR}Ctrl C${RES} ${YELLOW_COLOR}组合按钮终止操作${RES}"
            echo -ne "\r\n ${SHAN}按回车继续更新${RES}"
            read
            echo -e "\r\n${GREEN_COLOR}停止 Alist 进程${RES}"
            systemctl stop alist
            rm -rf /opt/alist/*
            VERSION=v3
            INSTALL_V3
            INIT >/dev/null 2>&1
            echo -e "\r\n${GREEN_COLOR}启动 Alist 进程${RES}"
            systemctl start alist
            echo -e "\r\n${GREEN_COLOR}Alist 已更新到最新稳定版！${RES}\r\n"
            exit 0;
        fi

        echo -e "${GREEN_COLOR}停止 Alist 进程${RES}\r\n"
        systemctl stop alist

        # 备份 alist 二进制文件，供下载更新失败回退
        mv /opt/alist/alist /tmp/alist.bak
        echo -e "${GREEN_COLOR}下载 Alist $latest_version3 ...${RES}"
        curl -Lk "$mirror"https://github.com/alist-org/alist/releases/download/"$latest_version3"/alist-linux-amd64.tar.gz -o /tmp/alist.tar.gz $CURL_BAR
        tar zxf /tmp/alist.tar.gz -C /opt/alist/
        if [ ! -f /opt/alist/alist ]; then
            echo -e "${RED_COLOR}下载 alist-linux-$ARCH.tar.gz 出错，更新失败！${RES}"
            echo "回退所有更改 ..."
            mv /tmp/alist.bak /opt/alist/alist
            systemctl start alist
            exit 1;
        fi

        echo -e "\r\n${GREEN_COLOR}启动 Alist 进程${RES}"
        systemctl start alist
        echo -e "\r\n${GREEN_COLOR}Alist 已更新到最新稳定版！${RES}\r\n"

        # 删除临时文件
        rm -rf /tmp/alist*
    fi
)

# CURL 进度显示
if curl --help | grep progress-bar >/dev/null 2>&1; then # $CURL_BAR
    CURL_BAR="--progress-bar";
fi

# The temp directory must exist
if [ ! -d "/tmp" ]; then
    mkdir -p /tmp
fi

# Fuck bt.cn (BT will use chattr to lock the php isolation config)
chattr -i -R /opt/alist >/dev/null 2>&1

echo
echo -e " 稳定版：Alist 项目推送的 Release 版本\r\n 开发版：Alist 项目最新源码编译安装，安装耗时较长（要求可用 RAM 2GB 或更多）"
echo
echo -e "> 请选择："
echo
echo -e " 1 - 安装 Alist 稳定版 v1（版本：\e[1;36m$latest_version\e[0m EOL）"
echo -e " 2 - 安装 Alist 稳定版 v2（版本：\e[1;36m$latest_version2\e[0m EOL）"
echo -e " 3 - 安装 Alist 稳定版 v3（最新版本：\e[1;36m$latest_version3\e[0m）"
echo -e " 4 - 安装 Alist 开发版 v3（最新 commit：\e[1;36m$REMOTE_ALIST_COMMIT\e[0m）"
echo
echo -e " 5 - 卸载 Alist"
echo
echo -e " 6 - 重置配置"
if [ -f "/opt/alist/alist" ]; then
    echo -e " 7 - 更新（更新 Alist 到最新 v3 稳定版）"
fi
if [ -f "/opt/alist/alist" ]; then
    local_version=$(/opt/alist/alist -version 2>&1)
    if [ $? -eq 1 ]; then
        local_version=$(/opt/alist/alist version 2>&1)
    fi
    if [[ "$local_version" =~ "v2" ]]; then
        local_alist_version=`echo "$local_version" | grep v2 | awk '{print $2}'`
        local_go_version=`echo "$local_version" | grep "Go Version" | awk -F: '{print $2}'`
        local_version_commit=`echo "$local_version" | grep Commit | awk -F: '{print $2}'`
        pushd /opt/alist/ >/dev/null 2>&1
            get_password=`./alist -password 2>&1`
            local_password=`echo $get_password | awk -F'password: ' '{print $2}'`
        popd >/dev/null 2>&1
        echo
        echo -e " > 已安装版本：\e[1;36m$local_alist_version$local_version_commit$local_go_version\e[0m"
        echo -e "   管理密码：\e[1;36m$local_password\e[0m"
    elif [[ "$local_version" =~ "v3" ]]; then
        local_alist_version=`echo "$local_version" | tail -2 | grep Version | head -1 | awk '{print $2}'`
        local_go_version=`echo "$local_version" | grep "Go Version" | awk -F: '{print $2}'`
        local_version_commit=`echo "$local_version" | grep Commit | awk -F: '{print $2}'`
        pushd /opt/alist/ >/dev/null 2>&1
            get_password=`./alist password 2>&1 | tail -2`
            local_username=`echo "$get_password" | awk '{print $2}' | head -1`
            local_password=`echo "$get_password" | awk '{print $2}' | tail -1`
        popd >/dev/null 2>&1
        echo
        echo -e " > 已安装版本：\e[1;36m$local_alist_version$local_version_commit$local_go_version\e[0m"
        echo -e "   用户名：\e[1;36m$local_username\e[0m"
        echo -e "   密码：\e[1;36m$local_password\e[0m"
    else
        local_alist_version=`/opt/alist/alist -version | awk -F: '{print $2}'`
        echo
        echo -e " > 当前已安装版本：\e[1;36m$local_alist_version\e[0m"
    fi
fi
echo
echo -n "请输入："
read mode
case $mode in
[1]|[1-7]) ;;
*) echo -e "\n ${RED_COLOR}...输入错误，终止脚本${RES}\r\n";exit 1;;
esac
if [ -z $mode ]; then
    echo -e "\n ${RED_COLOR}...输入错误，终止脚本${RES}\r\n";exit 1;
else
    if [[ $mode == "1" ]]; then
        CHECK
        VERSION=v1
        INSTALL
        if [ "$?" == "3" ]; then
            exit 0;
        fi
        INIT
        if [ -f "/opt/alist/alist" ]; then
            SUCCESS
        else
            echo -e "${RED_COLOR} 安装失败${RES}"
        fi
    elif [[ $mode == "2" ]]; then
        CHECK
        VERSION=v2
        INSTALL_V2
        if [ "$?" == "3" ]; then
            exit 0;
        fi
        INIT
        if [ -f "/opt/alist/alist" ]; then
            SUCCESS
        else
            echo -e "${RED_COLOR} 安装失败${RES}"
        fi
    elif [[ $mode == "3" ]]; then
        CHECK
        VERSION=v3
        INSTALL_V3
        if [ "$?" == "3" ]; then
            exit 0;
        fi
        INIT
        if [ -f "/opt/alist/alist" ]; then
            SUCCESS
        else
            echo -e "${RED_COLOR} 安装失败${RES}"
        fi
    elif [[ $mode == "4" ]]; then
        CHECK
        VERSION=v3
        INSTALL_V3_BUILD
        INIT
        if [ -f "/opt/alist/alist" ]; then
            SUCCESS
        else
            echo -e "${RED_COLOR} 安装失败${RES}"
        fi
    elif [[ $mode == "5" ]]; then
        UNINSTALL
    elif [[ $mode == "6" ]]; then
        if [ -f "/opt/alist/data/config.json" ]; then
            echo
            echo -e " 正在重置数据库信息 ..."
            sleep 2
            rm -f /opt/alist/data.db
            systemctl restart alist
            echo
            exit 0;
        else
            INIT
        fi
    elif [[ $mode == "7" ]]; then
        UPDATE
    fi
fi
